#!/usr/bin/perl -w
#----------------------------------------------------------
# apply_conf_form.pl
# part of webconf-package for EIS/FAIR
# last modified: 2003/12/16
# (c) 2002-2003 by Christian Wuerdig (eisfair@chriswue.de)
#----------------------------------------------------------

use strict;
use CGI::Carp qw(fatalsToBrowser);
use Fcntl ':flock';
use syntax;
use formtools;
use parse_form;

use renderer_file;
use renderer_html;
use render_config;

our $VERBOSE;

no strict "subs";
do "conf.pl";
use strict "subs";

my $what = $ARGV[0];
my $tmpcnf = $ARGV[1];
my $configfile = $ARGV[2];
my $tmpform = $ARGV[3];
my $applyfile = $configfile;

my $Apply_Path = "/var/install/config.d";
my $Config_Path = "/etc/config.d";
my $Temp_Path = "/var/config/tmp";

my $classname = "result";
my $class = "class=\"$classname\"";

my $creation_time = localtime(time);

# read the temporary config
open(TMPCNF, "<$Temp_Path/$tmpcnf") || die("Could not open '$tmpcnf' for reading, reason: $!");
my @config = <TMPCNF>;
close(TMPCNF);

my @regex_list;
my @err_list;
# read the match-regex and error messages
open(TMPFORM, "<$Temp_Path/$tmpform") || die("Could not open '$tmpform' for reading, reason: $!");
while(<TMPFORM>) {
   s/\n$//;
   if (/-err=/) { push(@err_list, $_); }
   else { push(@regex_list, $_); }
}
close(TMPFORM);

my %config;
my $name;
my $val;
my $err;
my $aref;
my $formfile = $configfile;
foreach (@config) {
   ($name, $val) = split(/=/, $_, 2);
   $val =~ s/\r?\n//g;
   $val =~ s/^'|'$//g;
   # with type=REF it can be possible that we have more than one value for the same variable
   if ($config{"$name"}) { $config{"$name"} .= " $val"; }
   else { $config{"$name"} = $val; }
}

if ($what eq "reload") {
   my $cf = "$Config_Path/$configfile";
   my $t = `cp $cf $cf.webconf.old`;

   my $helpfile = $configfile;
   my $Help_Path = "/var/install/help";
   my $helpfound = -f "$Help_Path/$helpfile" ? 1 : 0;

   ($err, $aref) = parse_confform($configfile, $formfile);
   if ($err) { die "A fatal error occured while parsing the formfile! Exiting now."; }

   ### build up hashtable for rendering configfiles
   my $renderer = create_renderer_file($configfile);
   render_config($renderer, $aref, \%config);

   my %opt_hash = get_confvar($configfile);

   print_head_min("config-file formular", "class=\"confform\"");
   $renderer = create_renderer_html($configfile, "confform");
   render_config($renderer, $aref, \%config); 
  
   $t = `cp $cf.webconf.old $cf`;
   unlink("$cf.webconf.old", $tmpcnf);
}
else {    # apply
   # substitute predefined regexp's (defined in syntax.pm)
   @regex_list = subst_preregex(@regex_list);

   my %regex;
   my %err;
   # build regex-hash
   foreach (@regex_list) {
      ($name, $val) = split(/=/, $_, 2);
      $regex{"$name"} = $val;
   }
   
   #build errormessage-hash
   foreach (@err_list) {
      ($name, $val) = split(/=/, $_, 2);
      $name =~ s/-err$//;
      $err{"$name"} = $val;
   }
   
   print_head_min("applying form", $class);

   print "<pre $class>\n";
   print "checking Options with regexp's ... " if ($VERBOSE eq "yes");

   # check Options with regexp's
   my $failed = 0;
   my $yes = 0;
   my $target = "";
   use re 'taint';
   use re 'eval';
   foreach (keys(%regex)) {
      if ($config{"$_"} !~ /$regex{"$_"}/) {
         $failed = 1;
         $err{"$_"} =~ s/\%v/$config{$_}/;
         print "\n<b $class>Error</b> in <b $class>$_</b>: ", $err{"$_"};
      } elsif ($yes == -1) {
         $failed = 1; $yes = 0;
         print "\n<b $class>Error</b> in <b $class>$_</b>: Directory '$target' doesn't exist!";
      } elsif ($yes == -2) {
         $failed = 1; $yes = 0;
         print "\n<b $class>Error</b> in <b $class>$_</b>: Directory '$target' mustn't exist!";
      } elsif ($yes == -3) {
         $failed = 1; $yes = 0;
         print "\n<b $class>Error</b> in <b $class>$_</b>: File '$target' doesn't exist!";
      } elsif ($yes == -4) {
         $failed = 1; $yes = 0;
         print "\n<b $class>Error</b> in <b $class>$_</b>: File '$target' mustn't exist!";
      }
   }
   no re 'eval';
   no re 'taint';

   if (!$failed) {
      print "done\n" if ($VERBOSE eq "yes");
      print "saving config ... " if ($VERBOSE eq "yes");

      ### build up hashtable for rendering configfiles
      ($err, $aref) = parse_confform($configfile, $formfile);
      if ($err) { die "A fatal error occured while parsing the formfile! Exiting now."; }
      my $renderer = create_renderer_file($configfile);
      render_config($renderer, $aref, \%config);

      print "done<br>\n" if ($VERBOSE eq "yes");
      print "applying config ... \n" if ($VERBOSE eq "yes");
      if (-f "$Apply_Path/$applyfile") {
         my $output = `$Apply_Path/$applyfile`;
         print "$output\n";
      }
      elsif (-f "$Apply_Path/$applyfile.sh") {
         my $output = `$Apply_Path/$applyfile.sh`;
         print "$output\n";
      }
      else { print "no skript found<br>\n" if ($VERBOSE eq "yes"); }
      print ".done\n" if ($VERBOSE eq "yes");
   }
   else { print "\nconfig <b $class>NOT</b> saved due to Syntaxerrors!<br/>\n"; }
   
   unlink("$Temp_Path/$tmpcnf");

   print "</pre>\n";
   print "</body></html>\n";
}

