package print_form;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(print_confform print_servform);

#----------------------------------------------------------
# modul print_form.pm
# part of the webconf-package for eisfair
# $Id: print_form.pm,v 1.4 2004/03/07 23:36:53 chriswue Exp $
# (c) 2002-2003 by Christian Wuerdig (eisfair@chriswue.de)
#----------------------------------------------------------

use strict;
use CGI::Carp qw(fatalsToBrowser);
use formtools;

our $VERBOSE;
our $CONF_LINE_LEN;
our $SERV_LINE_LEN;

no strict "subs";
do "conf.pl";
use strict "subs";

my $prep_path = "/var/install/prep";
my $temp_path = "/var/config/tmp";
my $empty_tr = "<tr><td><br></td></tr>\n";

my $tf;
my $MYFH;



##############################################################################
###################### subroutines for service formular ######################
##############################################################################



# get_source description {{{
########################################################################
# sub get_source($sname, $href, $do_subst)
# - get all lines within "sourcerange" from source file $sname
# IN:  $sname    - name of source file
#      $range    - sourcerange (from-to)
#      $do_subst - remove all "\r?\n" from the lines
# OUT: $aref     - reference on an array with the lines from source
#                  or "" if no lines were found
########################################################################
# }}}
# sub get_source($sname, $range, $do_subst) {{{
sub get_source {
   my ($sname, $range, $do_subst) = @_;
   my @item;

   if (-f $sname) {
      my ($srcfrom, $srcto) = (0, 0);
      
      open(SOURCE, "<$sname");

      if    ($range =~ /^0$/)                     { while (<SOURCE>) { s/\r?\n//g if ($do_subst); push(@item, $_); }; goto EOS; }
      elsif ($range =~ /^([1-9]\d*)-x/)           { $srcfrom = $1 - 1; $srcto = -1; }
      elsif ($range =~ /^([1-9]\d*)-([1-9]\d*)$/) { $srcfrom = $1 - 1; $srcto = $2; }
      
      while(<SOURCE>) {
         next if ($srcfrom-- > 0 && $srcto--);
         s/\r?\n//g if ($do_subst);
         push(@item, $_);
         last if (!$srcto--);
      }
      
EOS:  close(SOURCE);
   }
   
   if (@item) { return \@item; }
   else { return ""; }
}
# }}}

# print_servform description {{{
#########################################################################
# sub print_servform($href, $classname)
# - prints a formular for the service administration
# IN:  $href      - reference on the form-structure (see parse_servform)
#      $classname - name for the CSS-class
# OUT: -
#########################################################################
# }}}
# sub print_servform($href, $classname) {{{
sub print_servform {
   my ($form_file, $href, $classname) = @_;

   my $creation_time = localtime(time);
   
   my %class;
   $class{"class"} = "class=\"$classname\"";
   $class{"input_button"} = "class=\"button-$classname\"";
   $class{"input_radio"} = "class=\"radio-$classname\"";
   $class{"input_check"} = "class=\"check-$classname\"";
   $class{"input_text"} = "class=\"text-$classname\"";
   $class{"td_optname"} = "class=\"optname-$classname\"";
   $class{"td_opt"} = "class=\"opt-$classname\"";

   $tf = ".tmpformfile.$href->{formfile}";
   $MYFH = "$temp_path/$tf";

   my $formname = $href->{"name"};
   my $prep = $href->{"prep"};
   
   # open the temp file for regex-matches and error-messages
   #no strict "subs";
   #sysopen(MYFH, "$MYFH", O_RDWR | O_TRUNC | O_CREAT, 0600) || die("Couldn't open '$MYFH', reason: $!\n");
   #use strict "subs";
   my $old_umask = umask(0077);
   open(MYFH, ">$MYFH") || die("Couldn't open '$MYFH', reason: $!\n");
   umask($old_umask);

   # print the head of the formular
   print_head($formname, $formname, $class{"class"}, 1);
   add_history("$formname#make_service_forms.pl?$form_file");
   print_history;
   print "<hr $class{class}>\n";
   print "<p><br><br><br></p>\n";
   
   # execute the prepare-script
   my $tmp = `$prep_path/$prep` if ($prep);

   my ($i, $j, $target, $subform_name, $finish, $hf, $hn);
   my ($vname, $match, $err, $bname);
   # loop over all subforms
   foreach $i (0 .. $#{ $href->{"subforms"} }) {
      $target = 'target="Result"';     # assume target="frame"
      $subform_name = $href->{"subforms"}[$i]{"name"};
      if ($subform_name eq "empty") { print "<p><br></p>\n"; }
      else {
         if    ($href->{"subforms"}[$i]{"target"} =~ /new/i)   { $target = 'target="ScriptResult"'; }
         elsif ($href->{"subforms"}[$i]{"target"} =~ /form/i)  { $target = 'target="Edit"'; }
         
	 $href->{"subforms"}[$i]{"script"} =~ s/#/%23/g;
         print "<form $class{class} action=\"apply_service_form.pl?", $href->{"subforms"}[$i]{"script"};
         print "+$tf\" method=\"post\" $target>\n";
         print "<fieldset $class{class}><legend $class{class}>$subform_name</legend>\n" if ($subform_name);
         print "<table $class{class}>\n";

         # loop over all options in a subform
         foreach $j (0 .. $#{ $href->{"subforms"}[$i]{"options"} }) {
	    $finish = 1;
            # create the form-entries for the different types
	    if ($href->{"subforms"}[$i]{"options"}[$j]{"name"} eq "empty") { 
	       print "$empty_tr";
	       $finish = 0;
	    }
            elsif ($href->{"subforms"}[$i]{"options"}[$j]{"type"} =~ /text/i) {
               print_serv_text($i, $j, \%class, $href);
	       $finish = 0;
            }
            elsif ($href->{"subforms"}[$i]{"options"}[$j]{"type"} =~ /line/i) {
               print_serv_line($i, $j, \%class, $href);
            }
            elsif ($href->{"subforms"}[$i]{"options"}[$j]{"type"} =~ /select|select_m/i) {
               print_serv_select($i, $j, \%class, $href);
            }
            elsif ($href->{"subforms"}[$i]{"options"}[$j]{"type"} =~ /choice|checkbox/i) {
               print_serv_choice($i, $j, \%class, $href);
            }
            else {    # assume type="field"
               print_serv_field($i, $j, \%class, $href);
            }
            
            # finish the form-element
            # print regex-field and help-link only if type is not text and name is not "empty"
            if ($finish) {
               $hf = $href->{"help"};
               $hn = ($href->{"subforms"}[$i]{"options"}[$j]{"hname"}) if ($hf);
               print "<a $class{class} href=\"show_help.pl?$hf+$hn\" target=\"Result\">Help</a>\n" if ($hn);
               
               $vname = $href->{"subforms"}[$i]{"options"}[$j]{"vname"};
               $match = $href->{"subforms"}[$i]{"options"}[$j]{"match"};
               $err = $href->{"subforms"}[$i]{"options"}[$j]{"errormsg"};
               
               print MYFH "$vname=$match\n";
               print MYFH "$vname-err=$err\n";
               
               print "</td></tr>\n";
            }
         }
         # close the subform
         print "</table>\n";
         print "</fieldset>\n" if ($subform_name);
         $bname = $href->{"subforms"}[$i]{"button"};
         print "<br><input $class{input_button} type=\"submit\" value=\" $bname \"><br>\n";
         print "</form>\n";
      }
   }

   # close the form
   #my $last_loc = `cat /var/config/tmp/.back`;
   #$last_loc =~ s/\r?\n//gc
   print "<p><br><br><br></p>\n";
   print "<hr $class{class}>\n";
   
   print_history;
   
   #print "<br><a $class{class} href=\"$last_loc\" target=\"_parent\">back</a>\n";
   #print "<br>return to <a $class{class} href=\"../index.html\" target=\"_parent\">Mainmenu</a>\n";

   print "<hr $class{class}>\n";
   print "<span $class{class}>created by make_service_form.pl on $creation_time.</span>\n" if ($VERBOSE eq "yes");
   print "</body></html>\n";
   
   # close temp file for regex-matches and error-messages
   close(MYFH);
}
# }}}

# print_serv_text description {{{
####################################################################
# sub print_serv_text($i, $j, $classref, $href)
# - prints a paragraph with some text (sorce given in the formfile)
# IN:  $i        - actual subform
#      $j        - actual option in subform
#      $classref - reference on hash with the CSS-classnames
#      $href     - reference on the form-structure
# OUT: -
####################################################################
# }}}
# sub print_serv_text($i, $j, $classref, $href) {{{
sub print_serv_text {
   my ($i, $j, $classref, $href) = @_;

   # check if a sourcefile is given
   my $sname = $href->{"subforms"}[$i]{"options"}[$j]{"source"};

   # text-align
   my $align = "left";     # assume align="left"
   if ($href->{"subforms"}[$i]{"options"}[$j]{"align"}) { 
      $align = lc($href->{"subforms"}[$i]{"options"}[$j]{"align"}); 
      if ($align eq "block") { $align = "justify"; }
   }
      
   # create the form-element
   print "</table>\n";
   print "<p $classref->{class} style=\"text-align:$align\">\n";
   
   my $aref = get_source($sname, $href->{"subforms"}[$i]{"options"}[$j]{"sourcerange"}, 0);
   if ($aref) { print @$aref; }
   else { print $href->{"subforms"}[$i]{"options"}[$j]{"text"}, "\n"; } 
   
   print "</p>\n";
   print "<table ", $classref->{"class"}, ">\n";
}
# }}}

# print_serv_line description {{{
################################################################
# sub print_serv_line($i, $j, $classref, $href)
# - prints a one-line input field (sorce given in the formfile)
# IN:  $i        - actual subform
#      $j        - actual option in subform
#      $classref - reference on hash with the CSS-classnames
#      $href     - reference on the form-structure
# OUT: -
################################################################
# }}}
# sub print_serv_line($i, $j, $classref, $href) {{{
sub print_serv_line {
   my ($i, $j, $classref, $href) = @_;
   my $val = "";

   # check if a sourcefile is given
   my $sname = $href->{"subforms"}[$i]{"options"}[$j]{"source"};

   # get the line
   if (-f $sname) {
      my $line = $href->{"subforms"}[$i]{"options"}[$j]{"sourcerange"};
      open(SOURCE, "<$sname");
      while(<SOURCE>) {
         last if (!--$line);
      }
      $val = $_;
      close(SOURCE);
   }

   # get names
   my $optname = $href->{"subforms"}[$i]{"options"}[$j]{"name"};
   my $vname = $href->{"subforms"}[$i]{"options"}[$j]{"vname"};
   
   # create the form-element
   print "<tr $classref->{class}>\n";
   if ($optname) { print "<td $classref->{td_optname}>$optname\: </td>\n"; }
   else { print "<td></td>\n"; }

   my $size = $href->{"subforms"}[$i]{"options"}[$j]{"size"};
   print "<td $classref->{td_opt}>\n";
   print "<input $classref->{input_text} name=\"$vname\" type=\"text\" size=\"$size\" value=\"$val\">\n";
}
# }}}

# print_serv_field description {{{
#############################################################
# sub print_serv_field($i, $j, $classref, $href)
# - prints a input field (sorce given in the formfile)
# IN:  $i        - actual subform
#      $j        - actual option in subform
#      $classref - reference on hash with the CSS-classnames
#      $href     - reference on the form-structure
# OUT: -
#############################################################
# }}}
# sub print_serv_field($i, $j, $classref, $href) {{{
sub print_serv_field {
   my ($i, $j, $classref, $href) = @_;

   # check if a sourcefile is given
   my $sname = $href->{"subforms"}[$i]{"options"}[$j]{"source"};

   my $aref = get_source($sname, $href->{"subforms"}[$i]{"options"}[$j]{"sourcerange"}, 1);
   $href->{"subforms"}[$i]{"options"}[$j]{"item"} = [ @$aref ] if ($aref);

   # get names
   my $optname = $href->{"subforms"}[$i]{"options"}[$j]{"name"};
   my $vname = $href->{"subforms"}[$i]{"options"}[$j]{"vname"};

   # create the form-element
   print "<tr $classref->{class}>\n";
   if ($optname) { print "<td $classref->{td_optname}>$optname\: </td>\n"; }
   else { print "<td></td>\n"; }
   
   my ($sizex, $sizey) = (80, 10);
   my $size = $href->{"subforms"}[$i]{"options"}[$j]{"size"};
   if ($size =~ /(\d+)x(\d+)/) { $sizex = $1; $sizey = $2; }

   print "<td $classref->{td_opt}>\n";
   print "<textarea ", $classref->{"class"}, " name=\"$vname\" cols=\"$sizex\" rows=\"$sizey\">\n";
   foreach (@{ $href->{"subforms"}[$i]{"options"}[$j]{"item"} }) { print "$_\n"; } 
   print "</textarea>\n"
}
# }}}

# print_serv_select description {{{
#############################################################
# sub print_serv_select($i, $j, $classref, $href)
# - prints a select box (sorce given in the formfile)
# IN:  $i        - actual subform
#      $j        - actual option in subform
#      $classref - reference on hash with the CSS-classnames
#      $href     - reference on the form-structure
# OUT: -
#############################################################
# }}}
# sub print_serv_select($i, $j, $classref, $href) {{{
sub print_serv_select {
   my ($i, $j, $classref, $href) = @_;

   # check if a sourcefile is given
   my $sname = $href->{"subforms"}[$i]{"options"}[$j]{"source"};

   my $aref = get_source($sname, $href->{"subforms"}[$i]{"options"}[$j]{"sourcerange"}, 1);
   $href->{"subforms"}[$i]{"options"}[$j]{"item"} = [ @$aref ] if ($aref);

   # get names
   my $optname = $href->{"subforms"}[$i]{"options"}[$j]{"name"};
   my $vname = $href->{"subforms"}[$i]{"options"}[$j]{"vname"};

   # create the form-element
   print "<tr ", $classref->{"class"}, ">\n";
   if ($optname) { print "<td ", $classref->{"td_optname"}, ">$optname\: </td>\n"; }
   else { print "<td></td>\n"; }
   
   my $size = $href->{"subforms"}[$i]{"options"}[$j]{"size"};

   print "<td ", $classref->{"td_opt"}, ">\n";
   my $sm = $href->{"subforms"}[$i]{"options"}[$j]{"type"} =~ /select_m/i ? "multiple" : "";
   print "<select ", $classref->{"class"}, " name=\"$vname\" size=\"$size\" $sm>\n";
   
   foreach (@{ $href->{"subforms"}[$i]{"options"}[$j]{"item"} })
      { print "<option $classref->{class}>$_</option>\n"; }
   
   print "</select>\n"
}
# }}}

# print_serv_choice description {{{
####################################################################
# sub print_serv_choice($i, $j, $classref, $href)
# - prints checkboxes or radiobuttons (sorce given in the formfile)
# IN:  $i        - actual subform
#      $j        - actual option in subform
#      $classref - reference on hash with the CSS-classnames
#      $href     - reference on the form-structure
# OUT: -
####################################################################
# }}}
# sub print_serv_choice($i, $j, $classref, $href) {{{
sub print_serv_choice {
   my ($i, $j, $classref, $href) = @_;

   # check if a sourcefile is given
   my $sname = $href->{"subforms"}[$i]{"options"}[$j]{"source"};

   my $aref = get_source($sname, $href->{"subforms"}[$i]{"options"}[$j]{"sourcerange"}, 1);
   $href->{"subforms"}[$i]{"options"}[$j]{"item"} = [ @$aref ] if ($aref);

   # get names
   my $optname = $href->{"subforms"}[$i]{"options"}[$j]{"name"};
   my $vname = $href->{"subforms"}[$i]{"options"}[$j]{"vname"};

   # create the form-element
   print "<tr ", $classref->{"class"}, ">\n";

   my $inptype = $href->{"subforms"}[$i]{"options"}[$j]{"type"} =~ /choice/i ? "radio" : "checkbox";
   my $cl = $inptype eq "radio" ? $classref->{"input_radio"} : $classref->{"input_check"};
   
   # arrangement of the boxes
   my $sizex = 0;
   my $sizey = 0;
   my $size = $href->{"subforms"}[$i]{"options"}[$j]{"size"};
   if    ($size =~ /^h$/i)      { $sizey--; }
   elsif ($size =~ /^h(\d+)$/i) { $sizex = $1; }
   elsif ($size =~ /^v$/i)      { $sizex--; }
   elsif ($size =~ /^v(\d+)$/i) { $sizey = $1; }
   if (!$sizex && !$sizey) { $sizey--; }     # assume horizontal

   my ($k, $cols, $row, $t, $item, $itemidx);
   
   # print the form-element
   if ($sizex < 0) {   # vertical
      $itemidx = $#{ $href->{"subforms"}[$i]{"options"}[$j]{"item"} };
      if ($optname) { print "<td ", $classref->{"td_optname"}, ">$optname\: </td>\n"; }
      else { print "<td></td>\n"; }
      print "<td $classref->{td_opt}>\n";
      foreach $k (0 .. $itemidx - 1) {
         $item = $href->{"subforms"}[$i]{"options"}[$j]{"item"}[$k];
         print "<input $cl name=\"$vname\" type=\"$inptype\" value=\"$item\"> $item<br>\n"; 
      }
      $k = $itemidx;
      $item = $href->{"subforms"}[$i]{"options"}[$j]{"item"}[$k];
      print "<input $cl name=\"$vname\" type=\"$inptype\" value=\"$item\"> $item &nbsp;&nbsp;\n";
   }
   elsif ($sizey < 0) {   # horizontal
      if ($optname) { print "<td ", $classref->{"td_optname"}, ">$optname\: </td>\n"; }
      else { print "<td></td>\n"; }
      print "<td ", $classref->{"td_opt"}, ">\n";
      foreach (@{ $href->{"subforms"}[$i]{"options"}[$j]{"item"} })
         { print "<input $cl name=\"$vname\" type=\"$inptype\" value=\"$_\"> $_ &nbsp;&nbsp;\n"; } 
   }
   elsif ($sizex) {   # hx
      print "</table>\n";
      print "<table ", $classref->{"class"}, ">\n";
      print "<tr ", $classref->{"class"}, ">\n";
      if ($optname) { print "<td ", $classref->{"td_optname"}, ">$optname\: </td>\n"; }
      else { print "<td></td>\n"; }
      $k = 0; $row = 0;
      $itemidx = $#{ $href->{"subforms"}[$i]{"options"}[$j]{"item"} };
      while ($k <= $itemidx) {
         $t = 1;
         if ($row) { print "<tr ", $classref->{"class"}, "><td></td>\n"; }
         while ($t <= $sizex && $k <= $itemidx) {
            $item = $href->{"subforms"}[$i]{"options"}[$j]{"item"}[$k];
            print "<td $classref->{td_opt}><input $cl name=\"$vname\" type=\"$inptype\" value=\"$item\"> $item</td>\n";
            $k++; $t++;
         }
         $row++;
         if ($t == $sizex + 1) { print "</tr>\n"; }
      }
      while ($t <= $sizex) { print "<td></td>\n"; $t++; }
      print "<td $classref->{td_opt}>\n";
   }
   else {    # vy
      $itemidx = $#{ $href->{"subforms"}[$i]{"options"}[$j]{"item"} } + 1; # we need number of items here
      $cols = int($itemidx / $sizey) - 1;
      $row = 0;
      if ($itemidx % $sizey != 0) { $cols++; }
      $itemidx--;    # get real index
      print "</table>\n";
      print "<table ", $classref->{"class"}, ">\n";
      print "<tr ", $classref->{"class"}, ">\n";
      if ($optname) { print "<td ", $classref->{"td_optname"}, ">$optname\: </td>\n"; }
      else { print "<td></td>\n"; }
      while ($row < $sizey) {
         if ($row) { print "<tr ", $classref->{"class"}, "><td></td>\n"; }
         for ($t = 0; $t <= $cols; $t++) {
            if ($row + $t * $sizey <= $itemidx) {
               $item = $href->{"subforms"}[$i]{"options"}[$j]{"item"}[$row + $t * $sizey];
               print "<td ", $classref->{"td_opt"}, ">";
               print "<input $cl name=\"$vname\" type=\"$inptype\" value=\"$item\"> $item</td>\n"; 
            } else { print "<td></td>"; }
         }
         $row++;
         if ($row < $sizey) { print "</tr>\n"; }
      }
      print "<td ", $classref->{"td_opt"}, ">\n";
   }
}
# }}}

1;
