package save_config;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(save_config);

#-----------------------------------------------------------------------------
# modul save_config.pm
# part of the webconf-package for eisfair
# last modified: 2003/11/15
# (c) 2002-2003 by Maximilian Pasternak (maximilian.pasternak@bluescreen.de)
#-----------------------------------------------------------------------------

use strict;
use CGI::Carp qw(fatalsToBrowser);
use Fcntl ':flock';

our $VERBOSE;
our $CONF_LINE_LEN;
our $SERV_LINE_LEN;

no strict "subs";
do "conf.pl";
use strict "subs";

my $config_path = "/etc/config.d";

my $separator = "-" x 79;



##############################################################################
#################### subroutines for config-file          ####################
##############################################################################



##############################################################################
# sub save_config($aref, $href, $cf, $ishelp)
# - writes a new config-file
# IN:  $aref      - reference on the form-structure (see parse_confform)
#      $href      - reference on a hash with the options from the config-file
#      $cf        - name of configfile relative to /etc/config.d/
#      $ishelp    - indicates whether a help file exists or not
# OUT: -
##############################################################################
sub save_config {
   my $aref = shift;
   my $href = shift;
   my $cf = shift;
   my $ishelp = shift;

   my $creation_time = localtime(time);
   my $help = $ishelp ? $cf : "";

   # get the keys from opt-hash
   my @k = sort keys(%$href);

   # open the config file for writing the config
   open(CONF, ">$config_path/$cf") || die "Could not open '$config_path/$cf' for writing, reason: $!.\n";
   flock(CONF, LOCK_EX);
   
   if (!$!) {

   # print the head of the config
   print CONF "#$separator\n";
   print CONF "# configurationfile for $cf\n";
   print CONF "# created by apply_form.pl on $creation_time\n";
   print CONF "#$separator\n\n";

   # formfile found?
   if (@$aref) {
      # loop over all groups
      my $i;
      foreach $i (0 .. $#$aref) {
         if ($aref->[$i]{"name"} eq "empty") { print "\n"; }
         else {
            # start group
            my $name = $aref->[$i]{"name"};

            print CONF "#$separator\n" if ($name);
            print CONF "# $name\n" if ($name);
            print CONF "#$separator\n\n";


            # loop over all options in a group
            my $j;
            foreach $j (0 .. $#{ $aref->[$i]{"options"} }) {
               my $master = $aref->[$i]{"options"}[$j]{"master"} || "";
               if    ($aref->[$i]{"options"}[$j]{"name"} eq "empty") { print CONF "\n"; }
               elsif ($aref->[$i]{"options"}[$j]{"done"}) { }
               elsif ($master) {     # assume a variable option
                     write_conf_multi($i, $j, $aref, $href, $help);
               }
               else {    # single value
                     my $oname = $aref->[$i]{"options"}[$j]{"name"};
                     my $val = $href->{"$oname"};
                     my $hint = $aref->[$i]{"options"}[$j]{"hint"};
                     
                     print CONF "$oname='$val'";
                     print CONF "\t\t# $hint" if ($hint);
                     print CONF "\n";
               }
            }
            
            # close group
            print CONF "\n\n";
         }
      }
   }
   else {    # no formfile found
   
      foreach (@k) { print CONF "$_='", $href->{$_},"'\n"; }
   }

   # close config file
   }

   flock(CONF, LOCK_UN);
   close(CONF);
}

###############################################################
# sub write_conf_multi($i, $j, $aref, $href, $cf)
# - writes all options belonging to the same master
# IN:  $i        - number of group
#      $j        - number of option in group
#      $aref     - reference on the form-structure
#      $href     - reference on the hash with the options
#      $cf       - name of the config-file
# OUT: -
###############################################################
sub write_conf_multi {
   my $i = shift;
   my $j = shift;
   my $aref = shift;
   my $href = shift;
   my $cf = shift;

   my $oname = $aref->[$i]{"options"}[$j]{"name"};
   my $master = $aref->[$i]{"options"}[$j]{"master"};
   my $match = $aref->[$i]{"options"}[$j]{"match"} || "";
   my $err = $aref->[$i]{"options"}[$j]{"errormsg"} || "Value '\%v' doesn't match '$match'";
   
   # quit if value of master in configfile is 0
   return if (!$href->{"$master"});
   
   my %m_hash;
   my $k;
   my $optcnt = $#{ $aref->[$i]{"options"} };
   # first: search all options in a group which have the same master
   foreach $k (0 .. $optcnt) {
      if ($aref->[$i]{"options"}[$k]{"master"}) {
         my $m = $aref->[$i]{"options"}[$k]{"master"};
         my $n = $aref->[$i]{"options"}[$k]{"name"};
         if ($m eq $master) { $m_hash{"$n"} = $m; $aref->[$i]{"options"}[$k]{"done"} = 1; }
      }
   }
   
   my %mm_hash;
   # remember the match- and type- and select-values of the options, because we need them for the output
   my %match_hash;
   my %err_hash;
   my %type_hash;
   my %val_hash;
   # second: get all options which have a master-master (they have a master in %m_hash)
   foreach $k (0 .. $optcnt) {
      if ($aref->[$i]{"options"}[$k]{"master"}) {
         my $m = $aref->[$i]{"options"}[$k]{"master"};
         if ($m_hash{"$m"}) { 
            my $n = $aref->[$i]{"options"}[$k]{"name"};
            $mm_hash{"$n"} = $m; 
            $match_hash{"$n"} = $aref->[$i]{"options"}[$k]{"match"} || "";
            $err_hash{"$n"} = $aref->[$i]{"options"}[$k]{"errormsg"} || "Value '\%v' doesn't match '$match_hash{$n}'";
            $type_hash{"$n"} = $aref->[$i]{"options"}[$k]{"type"};
            $val_hash{"$n"} = \@{ $aref->[$i]{"options"}[$k]{"val"} };
         }
      }
   }

   my $m_cnt = scalar(keys(%m_hash));
   my $mm_cnt = scalar(keys(%mm_hash));
   # loop over all masters in configfile
   my $c;
   foreach $c (1 .. $href->{"$master"}) {
      if ($m_cnt > 1 || $mm_cnt || $c == 1) { print CONF "\n"; }
      # loop over all options in a group
      foreach $k (0 .. $optcnt) {
         my $n = $aref->[$i]{"options"}[$k]{"name"};
         my $match = $aref->[$i]{"options"}[$k]{"match"} || "";
         my $err = $aref->[$i]{"options"}[$k]{"errormsg"} || "Value '\%v' doesn't match '$match'";
         my $old_n = $n;
         # option has single-master
         if ($m_hash{"$n"}) {
            $n =~ s/#/$c/;    # '#' indicates the position of the actual value of the master
	        my $content = $href->{"$n"} ? $href->{"$n"} : "";
            my $multitype = $aref->[$i]{"options"}[$k]{"type"};
            my $valref = $aref->[$i]{"options"}[$k]{"val"} ? \@{ $aref->[$i]{"options"}[$k]{"val"} } : "";
            print CONF "$n='$content'\n";
         } elsif ($mm_hash{"$n"} && !$aref->[$i]{"options"}[$k]{"done"}) {   # options has master-master
            my $m = $mm_hash{"$n"};
            my $old_m = $m;
            $m =~ s/#/$c/;
            # is value of sub-master in configfile > 0?
            if ($href->{"$m"}) {
               my $l;
               # loop over the value of sub-master in configfile
               foreach $l (1 .. $href->{"$m"}) {
		          if ($mm_cnt > 1 || $l == 1) { print "\n"; }
                  # get and print all options with the same master-master
                  my $key;
                  foreach $key (keys(%mm_hash)) {
                     if ($mm_hash{"$key"} eq $old_m) {
			            # mark all processed options as "done"
			            my $t;
			            for $t (0 .. $optcnt) {
			               if ($aref->[$i]{"options"}[$t]{"name"} eq $key) { $aref->[$i]{"options"}[$t]{"done"} = 1; }
              			}
                        my $old_key = $key;
                        $key =~ s/##/$l/;    # '##' indicates the position of the actual value of the sub-master
                        $key =~ s/#/$c/;     # '#' indicates the position of the actual value of the master
            			my $content = $href->{"$key"} ? $href->{"$key"} : "";

                        print CONF "$key='$content'\n";
                     }
                  }
               }
            }
         }
      }
      if ($c < $href->{"$master"}) {
         # unmark the options with a master-master for the next turn
         my $t;
         for $t (0 .. $optcnt) {
	         delete($aref->[$i]{"options"}[$t]{"done"}) if ($mm_hash{$aref->[$i]{"options"}[$t]{"name"}});
         }
      }
   }
}

1;

